
#include "boot.h"
#include "memory_layout.h"
#include "BootFATX.h"
#include "BootIde.h"

#define XBOX_SMB_IO_BASE 0xC000
#define XBOX_SMB_HOST_ADDRESS       (0x4 + XBOX_SMB_IO_BASE)
#define XBOX_SMB_HOST_COMMAND       (0x8 + XBOX_SMB_IO_BASE)
#define XBOX_SMB_HOST_DATA          (0x6 + XBOX_SMB_IO_BASE)
#define XBOX_SMB_GLOBAL_ENABLE      (0x2 + XBOX_SMB_IO_BASE)

#define XBOX_PIC_ADDRESS 0x10

#define SMC_CMD_POWER 0x02
#define SMC_SUBCMD_POWER_OFF 0x80

#define SMC_SUBCMD_POWER_RESET 0x01

static char *banks[8]={
	"Bank 0",
	"Bank 1",
	"Bank 2",
	"Bank 3",
	"Bank 4",
	"Bank 5",
	"Bank 6",
	"Bank 7",
};

static char *main_menu[]={
	"Enable Network Flashing",
	"Load BIOS from HDD (C:\\Bios)",
	"Load BIOS from Disc",
	"Unlock HDD",
	"Lock HDD",
};

int NiceMenu(char **alts, int nAlts);
void HddFlash(void);
void UnlockHDD(void);
void LockHDD(void);
int CDFlash(void);

int FATXListDir(FATXPartition *partition, int clusterId, char **res, int reslen, char *prefix);
int FATXFindDir(FATXPartition *partition, int clusterId, char *dir);
extern void etherboot(void);

void FlashBIOS(void){
	int n;

	while (1) {
		n = NiceMenu (main_menu, 5);

		switch (n) {
		case 0:
			etherboot();
			break;
		case 1:
			HddFlash();
			break;
		case 2:
			CDFlash();
			break;
		case 3:
			UnlockHDD();
			break;
		case 4:
			LockHDD();
			break;
		}
	}
}

int LockUnlockHDD(int nIndexDrive, int command){
	BYTE baMagic[0x200], baKeyFromEEPROM[0x10], baEeprom[0x30];
	int nVersionHashing;
	unsigned uIoBase;
	DWORD BootHddKeyGenerateEepromKeyData(BYTE *eeprom_data,BYTE *HDKey);
				
	int nVersionSuccessfulDecrypt=0;

	nVersionHashing = 0;

	uIoBase = tsaHarddiskInfo[nIndexDrive].m_fwPortBase;
	
	// first 0x30 bytes from EEPROM image we picked up earlier
	memcpy(&baEeprom[0], &eeprom, 0x30);

	memset(&baKeyFromEEPROM,0x00,0x10);
	nVersionHashing = BootHddKeyGenerateEepromKeyData( &baEeprom[0], &baKeyFromEEPROM[0]);

	memset(&baMagic,0x00,0x200);

	// Calculate the hdd pw from EEprom and Serial / Model Number
	HMAC_SHA1 (&baMagic[2], baKeyFromEEPROM, 0x10,
			   tsaHarddiskInfo[nIndexDrive].m_szIdentityModelNumber,
			   tsaHarddiskInfo[nIndexDrive].m_length,
			   tsaHarddiskInfo[nIndexDrive].m_szSerial,
			   tsaHarddiskInfo[nIndexDrive].s_length);
	
	if (nVersionHashing == 0) {
		printk("Unable to generate password from EEPROM - corrupt?\n");
		return 0;
	}

	{ int i;
	VIDEO_ATTR=0xffc8c8c8;
	printk ("Key: ");
	VIDEO_ATTR=0xffc8c800;
	for (i = 0; i < 20; i++) {
		printk("%x", baMagic[i+2] & 0xff);
	}
	printk("\n");
	}
	
	return DriveSecurityChange(uIoBase, nIndexDrive, command, &baMagic[2]);
}

void UnlockHDD(void){
	printk("\n");
	printk ("Unlocking HDD...\n");
	LockUnlockHDD(0,IDE_CMD_SECURITY_DISABLE);

	// reset
  IoOutputWord(XBOX_SMB_HOST_ADDRESS, ((XBOX_PIC_ADDRESS) << 1));
  IoOutputByte(XBOX_SMB_HOST_COMMAND, SMC_CMD_POWER);
  IoOutputWord(XBOX_SMB_HOST_DATA, SMC_SUBCMD_POWER_RESET);
  IoOutputWord(XBOX_SMB_IO_BASE, IoInputWord(XBOX_SMB_IO_BASE));
  IoOutputByte(XBOX_SMB_GLOBAL_ENABLE, 0x0a);

	while (1)
		;
}
void LockHDD(void){
	printk("\n");
	printk ("Locking HDD...\n");
	LockUnlockHDD(0,IDE_CMD_SECURITY_SET_PASSWORD);
	
	/* reset */

  IoOutputWord(XBOX_SMB_HOST_ADDRESS, ((XBOX_PIC_ADDRESS) << 1));
  IoOutputByte(XBOX_SMB_HOST_COMMAND, SMC_CMD_POWER);
  IoOutputWord(XBOX_SMB_HOST_DATA, SMC_SUBCMD_POWER_RESET);
  IoOutputWord(XBOX_SMB_IO_BASE, IoInputWord(XBOX_SMB_IO_BASE));
  IoOutputByte(XBOX_SMB_GLOBAL_ENABLE, 0x0a);
        
	while (1)
		;
}

void HddFlash(){
	FATXPartition *partition;
	int bioscluster;
	int offset;
	char *alts[258];
	int n = 1;
	
	partition = OpenFATXPartition(0,SECTOR_SYSTEM,SYSTEM_SIZE);

	alts[0] = "Back to menu";

	// Messy, but fun!
	bioscluster = FATXFindDir(partition, FATX_ROOT_FAT_CLUSTER, "bios");
	if (bioscluster == -1 || bioscluster == 1) {
		bioscluster = FATXFindDir(partition, FATX_ROOT_FAT_CLUSTER, "Bios");
		if (bioscluster == -1 || bioscluster == 1) {
			bioscluster = FATXFindDir(partition, FATX_ROOT_FAT_CLUSTER, "BIOS");
			if (bioscluster == -1 || bioscluster == 1) {
				printk("No C:\\Bios found...\n");
				while(1);
			} else {
				n = FATXListDir(partition, bioscluster, &alts[1], 256, "C:\\BIOS\\");
			}
		} else {
			n = FATXListDir(partition, bioscluster, &alts[1], 256, "C:\\Bios\\");
		}
	} else {
		n = FATXListDir(partition, bioscluster, &alts[1], 256, "C:\\bios\\");
	}

	n = NiceMenu (alts, n+1);

	if (n > 0) {
		int res;
		FATXFILEINFO fileinfo;
		res = LoadFATXFilefixed(partition, alts[n]+2, &fileinfo, (BYTE*)0x100000);
		if (!res) {
			printk("Loading failed :(   Halting\n");
			while(1)
				;
		}
		printk("\n");
		printk ("Bios '%s', %dkbytes\n", alts[n]+2, fileinfo.fileSize >> 10);

		offset = 0;
#ifdef BANK_SEL
		switch (fileinfo.fileSize) {
		case 1024*1024:
			offset = 1024 * 1024 * NiceMenu(banks, 2);
			break;
		case 512*1024:
			offset = 512 * 1024 * NiceMenu(banks, 4);
			break;
		case 256*1024:
			offset = 256 * 1024 * NiceMenu(banks, 8);
			break;
		default:
			printk("BIOS must have size 256k, 512k or 1024k\n");
			while (1)
				;
		}
#endif
		res = BootReflashAndReset((BYTE*)0x100000,offset,fileinfo.fileSize);
		printk("flash failed: %d\n",res);
	}

	CloseFATXPartition(partition);

}

extern void ClearScreen (void);

int CDFlash(void){

	DWORD dwConfigSize=0;
	BYTE ba[2048];
	BYTE bCount=0, bCount1;
	unsigned int n;
	ISO_PRIMARY_VOLUME_DESCRIPTOR * pipvd;
	char sz[64];
	
	DWORD dwY=VIDEO_CURSOR_POSY;
	DWORD dwX=VIDEO_CURSOR_POSX;

	I2CTransmitWord(0x10, 0x0c00); // eject DVD tray
	wait_ms(2000); // Wait for DVD to become responsive to inject command
                        	
selectinsert:
	printk("\n\n");
	ClearScreen ();

	while(DVD_TRAY_STATE != DVD_CLOSING) {
	
		VIDEO_CURSOR_POSX=dwX;	
		VIDEO_CURSOR_POSY=dwY;
			
		bCount++;
		bCount1=bCount; if(bCount1&0x80) { bCount1=(-bCount1)-1; }
			VIDEO_ATTR=0xff000000|(((bCount1>>1)+64)<<16)|(((bCount1>>1)+64)<<8)|0 ;
		printk("\2Please insert CD - Flashing Mode\n\2");
		
		for (n=0;n<1000000;n++) {;}
	}


	VIDEO_ATTR=0xffffffff;
	printk("\n\n");
	ClearScreen ();

	// wait until the media is readable

	{
		bool fMore=true, fOkay=true;
		int timeoutcount = 0;
		
		while(fMore) {
			timeoutcount++;
			// We waited very long now for a Good read sector, but we did not get one, so we
			// jump back and try again
			if (timeoutcount>200) {
				I2CTransmitWord(0x10, 0x0c00); // eject DVD tray	
				wait_ms(2000); // Wait for DVD to become responsive to inject command
				goto selectinsert;
			}
			wait_ms(200);
			
			if(BootIdeReadSector(1, &ba[0], 0x10, 0, 2048)) { // starts at 16
				VIDEO_CURSOR_POSX=dwX;
				VIDEO_CURSOR_POSY=dwY;
				bCount++;
				bCount1=bCount; if(bCount1&0x80) { bCount1=(-bCount1)-1; }

				VIDEO_ATTR=0xff000000|(((bCount1)+64)<<16)|(((bCount1>>1)+128)<<8)|(((bCount1)+128)) ;

				printk("\2Waiting for drive\2\n");
	
			} else {  // read it successfully
				fMore=false;
				fOkay=true;
			}
		}

		if(!fOkay) {
			printk("cdrom unhappy\n");
			while(1);
		} else {
//			printk("\n");
		}
	}

	printk("\n\n");
	ClearScreen (); 
        
	{
		printk("\n");
	
		ISO_PRIMARY_VOLUME_DESCRIPTOR * pipvd = (ISO_PRIMARY_VOLUME_DESCRIPTOR *)&ba[0];
		char sz[64];
		memset(&sz,0x00,sizeof(sz));
		VIDEO_ATTR=0xffeeeeee;
		printk("CD-Rom Label: ");
		VIDEO_ATTR=0xffeeee00;
		BootIso9660DescriptorToString(pipvd->m_szVolumeIdentifier, sizeof(pipvd->m_szVolumeIdentifier), sz);
		printk("%s\n", sz);
	}
    
	{
		static const char * const filenames[]={"/BIOS.BIN","/LINUXBIO.BIN","/XECUTER.BIN","/LINUX.BIN","/CROMWELL.BIN","/BIOSUPD1.BIN",NULL};
		int len;
		int i=-1;
		do {
			i++;
			if(filenames[i]==NULL){
				printk("No Bios Update found... Halting!");
				while(1);
			}
			len=BootIso9660GetFile(1, filenames[i],
					       (BYTE *)0x100000,
					       0x100000, 0x0);
			
		} while (len<1024);
		
		
		VIDEO_ATTR=0xffc8c8c8;
		printk("Found BiosUpdate: ");
		VIDEO_ATTR=0xffc8c800;
		printk("%s (%dk)\n",filenames[i],len>>10);
		
		{
			int res = BootReflashAndReset((BYTE*)0x100000,0,len);
			printk("flash failed: %d\n",res);
		}
		
	}

	printk("   We are done flashing, didn't I reboot now?\n");

	return true;
}
